#!/usr/bin/perl -w

# Copyright 2017, 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Lingua::EN::Numbers;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'chamfer','transform_to_horizontal','xy_rotate_minus90';
use MyPrintwrap ':all';


my $level = $ARGV[0] || 3;
print "  % generated by tools/$FindBin::Script $level\n";

my $path = Math::PlanePath::CCurve->new;

sub run {
  my %options = @_;
  my $level  = $options{'level'};
  my $segments   = $options{'segments'};
  my $y_min  = $options{'y_min'};
  my $y_max  = $options{'y_max'};
  my $labels = $options{'labels'};

  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my $middle = $n_hi/2;
  my $n_start = $middle - $segments;
  my $n_end = $middle + $segments;

  my $transform = sub {
    my ($x,$y) = @_;
    return transform_to_horizontal($x,$y, $level);
  };

  $Printwrap = 2;
  my $rotate = (180 - 45*$level) % 360;
  printwrap("\\begin{scope}[shift=(pos-$level)"
            . ($level % 2 ? ",scale={1/sqrt(2)}" : "")
            . "]\n");

  $Printwrap = 4;
  my $arrow = ''; # ($level==0 ? '->,' : '');
  $Printwrap = 4;
  foreach my $n ($n_start .. $n_end-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    my $n_other = $path->xyxy_to_n($x2,$y2, $x,$y);

    ($x,$y) = $transform->($x,$y);
    ($dx,$dy) = $transform->($dx,$dy);
    ($x2,$y2) = $transform->($x2,$y2);

    next if defined $y_min && ! ($y > $y_min || $y2 > $y_min);
    next if defined $y_max && ! ($y < $y_max || $y2 < $y_max);

    $x  .= chamfer($dx,'\\MyChamfer');
    $y  .= chamfer($dy,'\\MyChamfer');
    $x2 .= chamfer(-$dx,'\\MyChamfer');
    $y2 .= chamfer(-$dy,'\\MyChamfer');

    if (defined $n_other && $n_other < $n_hi) {
      my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
      $x  .= chamfer($rx,'\\MyGap');
      $y  .= chamfer($ry,'\\MyGap');
      $x2 .= chamfer($rx,'\\MyGap');
      $y2 .= chamfer($ry,'\\MyGap');
      printwrap(" \\draw[my harpoon] ($x,$y)--($x2,$y2);");
    } else {
      printwrap(" \\draw[->] ($x,$y)--($x2,$y2);");
    }
  }
  printwrap_newline();

  $Printwrap = 2;
  printwrap("\\end{scope}\n");
  print "  % end generated\n";
}

my %level_to_options
  = (3 => { segments => 2, labels => { } },
     4 => { segments => 4 },
     5 => { segments => 6 },
     6 => { segments => 11 },
     7 => { segments => 21, y_min => 10 },
     8 => { segments => 43, y_min => 11 },
     9 => { segments => 80, y_min => 20, y_max => 26 },
    );

my $options = $level_to_options{$level};
run (level => $level, %$options);

exit 0;
