#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min','sum';
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()
use Tie::IxHash;

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'xy_rotate_plus90','xy_rotate_minus90','transform_to_horizontal';
use MyPrintwrap ':all';
use MyLineSegments;


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MyLevelSubOne}{",$level-1,"}\n";
my $path = Math::PlanePath::CCurve->new;

$Printwrap = 2;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub xydxdy_left_to_square_lower_left {
  my ($x,$y, $dx,$dy) = @_;
  my ($lx,$ly) = xy_rotate_plus90($dx,$dy); # to the left
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  my ($x3,$y3) = ($x+$lx, $y+$ly);
  my ($x4,$y4) = ($x2+$lx, $y2+$ly);
  return (min($x, $x2, $x3, $x4),
          min($y, $y2, $y3, $y4));
}
sub xydxdy_right_to_square_lower_left {
  my ($x,$y, $dx,$dy) = @_;
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy); # to the right
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  my ($x3,$y3) = ($x+$rx, $y+$ry);
  my ($x4,$y4) = ($x2+$rx, $y2+$ry);
  return (min($x, $x2, $x3, $x4),
          min($y, $y2, $y3, $y4));
}

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my $segments = MyLineSegments->new;
my %squares_seen;

$Printwrap = 2;
printwrap("\\fill[my grey] foreach \\p in {");
$Printwrap += 2;
my $Q = 0;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($x,$y) = transform($x,$y);
  ($dx,$dy) = transform($dx,$dy);

  $segments->add($x,$y, $x+$dx,$y+$dy);

  foreach my $func (\&xydxdy_left_to_square_lower_left,
                    \&xydxdy_right_to_square_lower_left) {

    my ($sx,$sy) = $func->($x,$y, $dx,$dy);
    if (! $squares_seen{"$sx,$sy"}++) {
      $Q++;
      printwrap("($sx,$sy),");
    }
  }
}
printwrap("} ");
printwrap("{ \\p +(\\MyGap,\\MyGap) rectangle +(1-\\MyGap,1-\\MyGap) };\n");


$Printwrap = 2;
printwrap("\\draw");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");

print "  \\newcommand{\\MyQ}{$Q}\n";
print "% GP-Test  Q($level) == $Q\n";

print "  % end generated\n";
exit 0;
