#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'mul','div','xy_rotate_minus90','transform_to_horizontal',
  '@dir4_to_dx', '@dir4_to_dy';
use MyPrintwrap ':all';


my $level = 13;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::CCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

# find QD4(13)=1 first 4-double square
my ($qd4_x,$qd4_y);
my @qd4_n_list;
N: foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);

  my ($x2,$y2) = ($x+$dx, $y+$dy);
  my ($x3,$y3) = ($x2+$rx, $y2+$ry);
  my ($x4,$y4) = ($x +$rx, $y +$ry);

  my @n_list = ($path->xyxy_to_n_list_either($x, $y,  $x2,$y2),
                $path->xyxy_to_n_list_either($x2,$y2, $x3,$y3),
                $path->xyxy_to_n_list_either($x3,$y3, $x4,$y4),
                $path->xyxy_to_n_list_either($x4,$y4, $x, $y));
  @n_list == 8 or next N;
  foreach my $other_n (@n_list) {
    $other_n < $n_hi or next N;
  }
  ($qd4_x,$qd4_y) = ($x,$y);
  @qd4_n_list = sort {$a<=>$b} $n, @n_list;
  last;
}
defined $qd4_x or die "Oops, QD4 not found in k=$level";
my ($qd4_tx,$qd4_ty) = transform($qd4_x,$qd4_y);
print "  % QD4 at N=",join(',',@qd4_n_list)," at $qd4_x,$qd4_y transformed $qd4_tx,$qd4_ty\n";

my $qd4_tx_min = $qd4_tx - 3;
my $qd4_tx_max = $qd4_tx + 5;
my $qd4_ty_min = $qd4_ty - 3;
my $qd4_ty_max = $qd4_ty + 5;

sub in_range {
  my ($tx1,$ty1, $tx2,$ty2) = @_;
  return ($qd4_tx_min <= $tx1 && $tx1 <= $qd4_tx_max
          && $qd4_ty_min <= $ty1 && $ty1 <= $qd4_ty_max
          && $qd4_tx_min <= $tx2 && $tx2 <= $qd4_tx_max
          && $qd4_ty_min <= $ty2 && $ty2 <= $qd4_ty_max
         );
}


$Printwrap = 2;
foreach my $x ($qd4_x - 20 .. $qd4_x + 20) {
  foreach my $y ($qd4_y - 20 .. $qd4_y + 20) {
    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];

      my ($x2,$y2) = ($x+$dx, $y+$dy);
      my $n = $path->xyxy_to_n($x,$y, $x2,$y2);
      next unless defined $n && $n < $n_hi;

      my ($tx1,$ty1) = transform($x,$y);
      my ($tx2,$ty2) = transform($x2,$y2);
      next unless in_range($tx1,$ty1, $tx2,$ty2);

      my $tdx = $tx2 - $tx1;
      my $tdy = $ty2 - $ty1;
      printwrap("\\pic at ($tx1,$ty1) [transform shape] {my line={$tdx,$tdy}};");
    }
  }
}
printwrap_newline();

print "  \\coordinate (start) at (0,0);\n";
{
  my ($x,$y) = mul($x_hi,$y_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}

{
  my ($x,$y) = transform($qd4_x,$qd4_y);
  print "  \\coordinate (QD4) at ($x,$y);\n";
}

print "  % end generated\n";
exit 0;
