#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyCCurve 'transform_to_horizontal';
use MyPrintwrap ':all';


my $level = 12;

print "% generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::CCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

print "\\tikzset{\n";
print "  my C curve fractal pic/.pic={\n";

my $scale = sqrt($x_hi**2 + $y_hi**2);
print "  \\begin{scope}[scale=1/$scale]\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

$Printwrap = 2;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
my $draw_count = $segments->lines_joined_count_points;
printwrap(";\n");

print "  \\end{scope}  % total drawn with $draw_count points\n";
print "  },\n";
print "}\n";

print "% end generated\n";
exit 0;
