#!/usr/bin/perl -w

# Copyright 2015 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

# Right boundary length diagonals across zig-zag

use 5.004;
use strict;
use FindBin;
use Math::BaseCnv 'cnv';
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'chamfer','xy_rotate_plus90';
use MyPrintwrap ':all';


my $max_level = 21;

print "% generated by tools/$FindBin::Script\n";
$|=1;  

sub Rdiag_by_path {
  my ($k) = @_;
  ### Rdiag_by_path: $k
  my $path = Math::PlanePath::CCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  my $straight = 0;
  my $diagonal = 0;

  my $is_visited = sub {
    my ($x,$y) = @_;
    foreach my $n ($path->xy_to_n_list($x,$y)) {
      if ($n <= $n_hi) { return 1; }
    }
    return 0;
  };

  my ($x,$y) = (0,0);
  my ($dx,$dy) = (1,0);
  until ($x == $x_hi && $y == $y_hi) {
    ### at: "$x,$y dir $dx,$dy     straight $straight diagonal $diagonal"
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    ### consider straight: "$x2, $y2"
    if ($is_visited->($x2,$y2)) {
      $straight++;
      ($x,$y) = ($x2,$y2);
      next;
    }

    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
    $x2 += $lx;
    $y2 += $ly;
    ### consider diagonal: "$x2, $y2"
    if ($is_visited->($x2,$y2)) {
      ### diagonal ...
      ($x,$y) = ($x2,$y2);
      $diagonal += 2;
      next;
    }

    ### left turn 90 ...
    $x2 = $x + $lx;
    $y2 = $y + $ly;
    $is_visited->($x2,$y2) or die;
    ($x,$y) = ($x2,$y2);
    ($dx,$dy) = ($lx,$ly);
    $straight++;
  }
  return ($straight, $diagonal);
}

my $var = $FindBin::Script;
$var =~ s/\.pl$//;
$var =~ tr/-/_/;

my @straight;
my @diagonal;
foreach my $k (0 .. $max_level) {
  my ($straight, $diagonal) = Rdiag_by_path($k);
  push @straight, $straight;
  push @diagonal, $diagonal;
}

printwrap("% GP-DEFINE  ${var}_RStotal = {[ ");
printwrap_indent("% GP-DEFINE    ");
foreach my $i (0 .. $#straight) {
  my $comma = ($i == $#straight ? '' : ',');
  printwrap("$straight[$i]$comma");
}
printwrap(" ]};\n");

printwrap_indent("");
printwrap("% GP-DEFINE  ${var}_RZtotal = {[ ");
printwrap_indent("% GP-DEFINE    ");
foreach my $i (0 .. $#diagonal) {
  my $comma = ($i == $#diagonal ? '' : ',');
  printwrap("$diagonal[$i]$comma");
}
printwrap(" ]};\n");

print "% end generated\n";
exit 0;
