#!/usr/bin/perl -w

# Copyright 2015, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::Libm 'hypot';
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyPrintwrap ':all';


my $parts = 12 + 4;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::CCurve->new;

sub form {
  my ($x) = @_;
  if ($x == 0) {
    return '0';
  }
  return sprintf '%.5f', $x;
}

my @dir_to_dx = (1, 1/sqrt(2), 0, -1/sqrt(2), -1, -1/sqrt(2), 0, 1/sqrt(2));
my @dir_to_dy = (0, 1/sqrt(2), 1, 1/sqrt(2), 0, -1/sqrt(2), -1, -1/sqrt(2));

my @points = ('++(1,0) coordinate[pos=.5,name=top]');
my $length = 1;
my $dir_forward = 7;
my $dir_reverse = 1;
my $total = 1;
foreach my $part (1 .. $parts) {
  $length /= sqrt(2);
  {
    my $dx = $dir_to_dx[$dir_forward] * $length;
    my $dy = $dir_to_dy[$dir_forward] * $length;
    my $rel = '++(' . form($dx) . ',' . form($dy) . ')';
    if ($part <= 4) {
      $rel .= " coordinate[pos=.5,name=part-$part-right]";
    }
    push @points, $rel;
    $total += hypot($dx,$dy);
  }
  {
    my $dx = $dir_to_dx[$dir_reverse] * $length;
    my $dy = $dir_to_dy[$dir_reverse] * $length;
    my $rel = '++(' . form($dx) . ',' . form($dy) . ')';
    if ($part <= 3) {
      $rel .= " coordinate[pos=.5,name=part-$part-left]";
    }
    unshift @points, $rel;
    $total += hypot($dx,$dy);
  }
$dir_forward = ($dir_forward - 1) % 8;
$dir_reverse = ($dir_reverse + 1) % 8;
}

  $Printwrap = 2;
printwrap("\\draw (0,0)");
$Printwrap += 2;
my $join = '--';
foreach my $p (@points) {
  printwrap(" $join $p");
  $join = '--';
}
printwrap(";\n");

print "  % total ",form($total),"\n";
print "  % end generated\n";
exit 0;
